#ifndef COEFFICIENTS_H
#define COEFFICIENTS_H 1
#include "vectorSpace.hpp"

namespace AG
{

	// Intersect two points
	//    if there is a match apply the functToCall on the value
	//
	template <typename VALUE_TYPE>
	Point<VALUE_TYPE> & intersect (Point<VALUE_TYPE> & p1, Point<VALUE_TYPE> & p2,
								   VALUE_TYPE (*funcToCall)(VALUE_TYPE, VALUE_TYPE))
	{
		// return point
		//
		Point<VALUE_TYPE> * p3 = new Point<VALUE_TYPE>();

		// sort them
		//
		p1.sort(); 	p2.sort();

		// size
		//
		size_t p1Size = p1.size();  size_t p2Size = p2.size();

		size_t i = 0; size_t j = 0; 
		while (i < p1Size && j < p2Size)
		{
			if (p1[i].first == p2[j].first) 
			{ 
				p3->push_back(p1[i].first, funcToCall(p1[i].second, p2[j].second));
				i++; j++;
			} 
			else if (p1[i].first < p2[j].first)
			{
				i++;
			}
			else 
			{
				j++;
			}
		}
		p3->setSorted();   // sorted by costruction
		return *p3;
	}


		// Intersect two points
	//    if there is a match apply the functToCall on the value
	//
	template <typename VALUE_TYPE>
	Point<VALUE_TYPE> & unione   (Point<VALUE_TYPE> & p1, Point<VALUE_TYPE> & p2,
								   VALUE_TYPE (*funcToCall)(VALUE_TYPE, VALUE_TYPE))
	{
		// return point
		//
		Point<VALUE_TYPE> * p3 = new Point<VALUE_TYPE>();

		// sort them
		//
		p1.sort(); 	p2.sort();

		// size
		//
		size_t p1Size = p1.size();  size_t p2Size = p2.size();

		size_t i = 0; size_t j = 0; 

		// keep the order
		//
		while (i < p1Size && j < p2Size)
		{
			if (p1[i].first == p2[j].first) 
			{ 
				p3->push_back(p1[i].first, funcToCall(p1[i].second, p2[j].second));
				i++; j++;
			} 
			else if (p1[i].first < p2[j].first)
			{
				p3->push_back(p1[i].first, p1[i].second);
				i++;
			}
			else 
			{
				p3->push_back(p2[j].first, p2[j].second);
				j++;
			}
		}
		while (i < p1Size)
		{
			p3->push_back(p1[i].first, p2[i].second);
			i++;
		}
		while (j < p1Size)
		{
			p3->push_back(p2[j].first, p2[j].second);
			j++;
		}
		p3->setSorted();   // sorted by costruction
		return *p3;
	}

	// Jaccard  || p1 intesect p2 || / || p1 union p2 ||
	//
	template <typename VALUE_TYPE>
	double jaccard (Point<VALUE_TYPE> & p1, Point<VALUE_TYPE> & p2)
	{
		if (p1.size() == 0 || p2.size() == 0)
			return 0;
		//
		p1.sort(); 	p2.sort();

		// size
		//
		size_t p1Size = p1.size();  size_t p2Size = p2.size();
		size_t intersectionPosAssigned = 0; size_t unionPosAssigned = 0;

		size_t i = 0; size_t j = 0; 
		while (i < p1Size && j < p2Size)
		{
			unionPosAssigned++; 

			if (p1[i].first == p2[j].first) 
			{ 
				intersectionPosAssigned++; i++; j++;
			} 
			else if (p1[i].first < p2[j].first)
			{
				i++;
			}
			else 
			{
				j++;
			}
		}
		while (i < p1Size) { unionPosAssigned++; i++; }
		while (j < p2Size) { unionPosAssigned++; j++; }
		
		std::cout << "insersectionPosAssigned = " << intersectionPosAssigned << std::endl;
		std::cout << "unionPosAssigned = " << unionPosAssigned << std::endl;

		return ((double) intersectionPosAssigned / unionPosAssigned);
	}

	//
	// Art1 cooef   || p1 intersect p2 || / || p1 || + \beta > || p2 || / \beta + num_features
 	//
	template <typename VALUE_TYPE>
	bool artTest (Point<VALUE_TYPE> & p1, Point<VALUE_TYPE> & p2,
				  double beta, unsigned int num_features)
	{
		if (p1.size() == 0 || p2.size() == 0)
			return 0;
		//
		p1.sort(); 	p2.sort();

		// size
		//
		size_t p1Size = p1.size();  size_t p2Size = p2.size();
		size_t intersectionPosAssigned = 0; 

		size_t i = 0; size_t j = 0; 
		while (i < p1Size && j < p2Size)
		{
			if (p1[i].first == p2[j].first) 
			{ 
				intersectionPosAssigned++; i++; j++;
			} 
			else if (p1[i].first < p2[j].first)
			{
				i++;
			}
			else 
			{
				j++;
			}
		}

		std::cout << "insersectionPosAssigned = " << intersectionPosAssigned << std::endl;

		/*		|| p1 intersect p2 || / || p1 || + \beta > || p2 || / \beta + num_features */
		return intersectionPosAssigned / ( p1.size() + beta) > p2.size() / (num_features + beta);
	}
};

#endif